/*
The MIT License (MIT)

Copyright (c) 2014 Markus Harms, ma@mh-svr.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE. 
 */
log( "Pong-Form", "load module");

function pongFormDivHTML( divId, resourceURL, params ) {
	log( "Pong-Form",  "divId="+divId+" resourceURL="+resourceURL );

	if ( moduleConfig[ divId ] != null ) {
		moduleConfig[ divId ].resourceURL = resourceURL;
		pongFormRenderHTML( divId, resourceURL, params, moduleConfig[ divId ]  );
	} else {
		$.getJSON( 
			resourceURL+"/pong-form", 
			function( pmd ) {
				moduleConfig[ divId ] = pmd;
				moduleConfig[ divId ].resourceURL = resourceURL;
				pongFormRenderHTML( divId, resourceURL, params, pmd );
			}
		);					
	}	
}

var checkboxNames = [];
var checkboxVals = [];

function pongFormRenderHTML( divId, resourceURL, params, pmd ) {
	log( "Pong-Form", "start '"+pmd.description+"'");
	var contentItems = [];
	contentItems.push( '<div id="'+divId+'PongFormDiv" class="pongFormFrm">' );
	contentItems.push( '<form id="'+divId+'PongFormFrm">' );
	if ( pmd.label != null ) {				
		contentItems.push( '<fieldset>' );
		contentItems.push( '<legend>'+ $.i18n( pmd.label )  +'</legend>' );
	}
	postLst = [];			
	checkboxNames = [];
	checkboxVals = [];
	 
	log( "Pong-Form", "start fields" );
	if ( pmd.fieldGroups != null ) {
		for ( var i = 0; i < pmd.fieldGroups.length; i++ ) {
			contentItems.push( '<div class="pongFormInputGrp'+i+'">' );
			var grp = pmd.fieldGroups[i];
			if ( grp.fieldset != null ) {
				contentItems.push( '<fieldset>' );
				contentItems.push( '<legend>'+ $.i18n( grp.fieldset )  +'</legend>' );				
			}
			for ( var j = 0; j < grp.columns.length; j++ ) {
				contentItems.push( '<div class="pongFormInputCol pongFormInput'+divId+'Grp'+i+'Col'+j+'">' );
				var col = grp.columns[j];
				if ( col.fieldset != null ) {
					contentItems.push( '<fieldset>' );
					contentItems.push( '<legend>'+ $.i18n( col.fieldset )  +'</legend>' );				
				}
				for ( var k = 0; k < col.formFields.length; k++ ) {
					var field = col.formFields[k];
					contentItems = contentItems.concat( pongFormRenderField( divId, field, 'G'+i+'C'+j ) );
					if ( field.type == 'checkbox' ) {
						if ( field.name != null ) { // normal checkbox operation
							log( "Pong-Form", "checkbox "+field.name  );
							if ( ! checkboxNames.inArray( field.name ) ) {
								log( "Pong-Form", "checkboxNames "+field.name  );
	
								checkboxNames.push( field.name ); 							
								postLst.push( field.name+": checkboxVal('"+field.name+"' )" );					
							}
							checkboxVals.push( { "name":field.name, "val":field.value, "id":divId+field.id } );
						} else { // stand alone checkbox
							log( "Pong-Form", "stand alone checkbox "+field.id  );
							postLst.push( field.id+": $( '#"+divId+field.id+":checked' ).val()" )						
						}
						// TODO handle check boch with same name !!
//							postLst.push( field.id+": $( '#"+divId+field.id+":checked' ).val()" )						
					} else if ( field.type != 'separator' && field.type != 'label' ) {
						postLst.push( field.id+": $( '#"+divId+field.id+"' ).val()" )						
					} 
				}
				if ( col.fieldset != null ) {
					contentItems.push( '</fieldset>' );
				}
				contentItems.push( '</div>' ); //pongFormInputGrpXColY	
			}
			if ( grp.fieldset != null ) {
				contentItems.push( '</fieldset>' );
			}
			contentItems.push( '</div>' ); //pongFormInputGrp
		}
	} else {
		log( "Pong-Form", "easy legacy two column form" );
		if ( pmd.formFields2 != null ) {
			log( "Pong-Form", "easy legacy two column form" );
			contentItems.push( '<div class="pongFormInput1">' );
			for ( var i = 0; i < pmd.formFields1.length; i++ ) {
				var field = pmd.formFields1[i];
				contentItems = contentItems.concat( pongFormRenderField( divId, field, 1 ) );
				postLst.push( field.id+": $( '#"+divId+field.id+"' ).val()" )
			}
			contentItems.push( '</div>' );				
		}
		if ( pmd.formFields2 != null ) {
			contentItems.push( '<div class="pongFormInput2">' );
			for ( var i = 0; i < pmd.formFields2.length; i++ ) {
				var field = pmd.formFields2[i];
				contentItems = contentItems.concat( pongFormRenderField( divId, field, 2 ) );
				postLst.push( field.id+": $( '#"+divId+field.id+"' ).val()" )
			}
			contentItems.push( '</div>' );						
		} 		
	}
	
	//postLst.push( field.id+": $( '#"+divId+field.id+":checked' ).val()" );					

	
	log( "Pong-Form", "start actions");
	contentItems.push( '<div class="pongFormActions">' );
	var initData = null;
	if ( pmd.actions != null ) {
		for ( var i = 0; i < pmd.actions.length; i++ ) {
			var action = pmd.actions[i];
			if ( action.onInit != null ) {
				initData = action.onInit;
			} else {
				contentItems = contentItems.concat( pongFormRenderAction( divId, action, postLst ) );			
			}
		}		
	}
	contentItems.push( '<div class="pongFormFrmField">' );
	
	contentItems.push( '</form>' );
	if ( pmd.label != null ) {				
		contentItems.push( '</fieldset>' );
	}
	contentItems.push( '</div>' );

	// output
	$( "#"+divId ).html( contentItems.join( "\n" ) );

	if ( initData != null ) {
		pongFormUpdateData( divId, initData );
	}
	log( "Pong-Form", "end.");
}


// select checkbox vals for GET/POST/...
function checkboxVal( name ) {
	log( "Pong-Form", "checkboxVal: "+name);
	var result = '[';
	var first = true;
	for ( var i = 0; i < checkboxVals.length; i++ ) {
		if ( checkboxVals[i].name == name  ) {
			if ( $( "#"+checkboxVals[i].id ).is(':checked') ) {
				if ( ! first ) { result += ','; }
				result += "'"+checkboxVals[i].val+"'";
				log( "Pong-Form", " val= "+checkboxVals[i].val);

				first = false;
			}
		}
	}
	result += ']';
	return result;
}

function pongFormRenderAction( divId, action, postLst ) {
	log( "Pong-Form", "action '"+action.actionName+"'");
	var contentItems = [];
	var method = "POST";
	if ( action.method != null ) { method = action.method; }
	
	if ( action.onChange != null ) { // no button, but a onChange event is done for the form

		var onChngSelector = '.'+divId+'PongFormField';
		if ( action.onChange != "*" ) {
			//log( "Pong-Form", "action.onChange: "+ action.onChange );
			
			var onCngElements = action.onChange.splitCSV();
			//log( "Pong-Form", "onCngElements= "+ onCngElements.join('|') );
			onChngSelector = '';
			for ( var i = 0; i < onCngElements.length; i++ ) {
				if ( i != 0 ) { onChngSelector += ', '; } 
				onChngSelector += '#'+divId+onCngElements[i];
			}
		}

		log( "Pong-Form", "selector: "+ onChngSelector );
		// do a aAJAX call on a change field event
		contentItems.push( '<script>' );
		contentItems.push( '  $( function() { ' );
		contentItems.push( '       $( "' +onChngSelector+ '" ).change(' );
		contentItems.push( '          function() { ' );
		contentItems.push( '              $.ajax( ' );
		contentItems.push( '                 { url: "'+action.actionURL+'", type: "'+method+'", ' );
		contentItems.push( '                   data: { '+ postLst +' } ' );
		contentItems.push( '              } ).done(  ' );
		contentItems.push( '                 function( dta ) { ' );
		if ( action.target != null ) {
			contentItems.push( '                       $( "#'+action.target+'Content" ).html( dta );  ' );						
		}
		if ( ( action.update != null ) && ( action.update.length != null ) ) {
			for ( var i = 0; i < action.update.length; i++ ) {
				contentItems.push( '                   udateModuleData( "'+action.update[i].resId+'Content", null ); ' );					
			}
		}
		contentItems.push( '                       return false;' ); 
		contentItems.push( '                  }  ' );
		contentItems.push( '              ); ');
		contentItems.push( '              return false;' ); 
		contentItems.push( '          }' );
		contentItems.push( '       ); ' );

		contentItems.push( '  } ); ' );
		contentItems.push( '</script>' );
		
	} else {

		// render a button
		contentItems.push( '<button id="'+divId+'Bt'+action.id+'">'+  $.i18n( action.actionName ) +'</button>' );
		contentItems.push( '<script>' );
		contentItems.push( '  $(function() { ' );
		contentItems.push( '       $( "#'+divId+'Bt'+action.id+'" ).click(' );
		contentItems.push( '          function() { ' );
//		contentItems.push( ' 	             alert( postLst );' );
		contentItems.push( '              $.ajax( { url: "'+action.actionURL+'", type: "'+method+'", ' );
		contentItems.push( '                 data: { '+ postLst +' } ' );
		contentItems.push( '              } ).done(  ' );
		contentItems.push( '                 function( dta ) { ' );
		if ( action.target != null ) {
			if ( action.target == '_parent' ) {
				contentItems.push( '                       window.location.replace( dta );');
			} else if ( action.target == 'modal' ) {
				contentItems.push( '                       alert( dta );  ' );
			} else {
				contentItems.push( '                       $( "#'+action.target+'Content" ).html( dta );  ' );
			}
		}
		if ( ( action.update != null ) && ( action.update.length != null ) ) {
			for ( var i = 0; i < action.update.length; i++ ) {
				contentItems.push( '                   udateModuleData( "'+action.update[i].resId+'Content", null ); ' );					
			}
		}
		contentItems.push( '                       return false;' ); 
		contentItems.push( '                  }  ' );
		contentItems.push( '              ); ');
		contentItems.push( '              return false;' ); 
		contentItems.push( '          }' );
		contentItems.push( '       ); ' );
		contentItems.push( '  }); ' );
		contentItems.push( '</script>' );

	}
	return contentItems;
}

function pongFormUpdateData( divId, paramsObj ) {
	log( "PoNG-Form",  'update '+divId );
	var def = moduleConfig[ divId ];
	log( "PoNG-Form", JSON.stringify( def ) );
	if ( def.resourceURL != null ) {
		$.getJSON( 
				def.resourceURL, 
				paramsObj,
				function( data ) { 	
					if ( def.dataDocSubPath == null ) {
						// table is the root of the doc
						log( "PoNG-Form",  'no tbl.dataDocSubPath' );
						// TODO add update code
						//alert( "Update form with dataDocSubPath..." );
						pongFormUpdateFieldsData( divId, data ); 
					} else {
						log( "PoNG-Form",  'tbl.dataDocSubPath='+def.dataDocSubPath );
						// table is somewhere in the DOM tree
						var pathToken = def.dataDocSubPath.split('.');
						log( "PoNG-Form",  'pathToken[0] ' + pathToken[0] );
						var subdata = data[ pathToken[0] ];
						for ( i = 1; i < pathToken.length; i++ ) {
							log( "PoNG-Form", 'pathToken['+i+'] ' + pathToken[i] );	
							subdata = subdata[ pathToken[i] ];
						}
						// console.log( ' subdata = ' + JSON.stringify( subdata ) );
						//poList[ divId ].pongListData = subdata;
						// TODO add update code
						//alert( "Update form w/o dataDocSubPath..." );
						pongFormUpdateFieldsData( divId, subdata ); 
					}
				} 
		);			
	} else {
		alert( "No Update!!" );
	}
}

function pongFormUpdateFieldsData( divId, dta ) {
	log( "PoNG-Form",  'pongFormUpdateFieldsData: DIV='+divId );
	var pmd = moduleConfig[ divId ];
	if ( pmd.fieldGroups != null ) {
		for ( var i = 0; i < pmd.fieldGroups.length; i++ ) {
			var grp = pmd.fieldGroups[i];
			for ( var j = 0; j < grp.columns.length; j++ ) {
				var col = grp.columns[j];
				for ( var k = 0; k < col.formFields.length; k++ ) {
					var field = col.formFields[k];
					var fieldId = '#'+divId+field.id; 
					if ( field.type == "text" ) {
						log( "PoNG-Form",  'pongFormUpdateFieldsData text: '+field.id+' '+dta[field.id] );
						if ( dta[field.id] != null ) {
							$( fieldId ).val( dta[field.id] );							
						} else {
							$( fieldId ).val( "" );
						}
					} else if ( field.type == "checkbox" ) {
						//TODO update checkbox value
					} else if ( field.type == "checkboxList" ) {
						//TODO update checkboxList value
					} else if ( field.type == "select" ) {
						log( "PoNG-Form",  'pongFormUpdateFieldsData select: '+field.id+' '+dta[field.id] );
						if ( dta[field.id] != null ) {
							$( fieldId ).val( dta[field.id] );							
						}else {
							$( fieldId ).val( -1 );
						}
						//TODO update select value
					} else {
						log( "Pong-Form", "ERROR '"+fieldDd+"': Can't update "+field.type);

					}

				}
			}
		}
	}
} 


function pongFormRenderField( divId, field, col ) {
	log( "Pong-Form", "field '"+field.id+"' ("+ field.type+")");
	var contentItems = [];
	contentItems.push( '<div class="pongFormField pongFormField'+col+'">' );
	
	if ( field.type == "separator" ) {
		contentItems.push( '<hr/>' );
	} else if ( field.type == "label" ) {
		contentItems.push( '<span class="pongFormLabelField">'+$.i18n( field.label )+'</span>' );
	} else {
		
		var title      = ""; if ( field.descr != null ) { title = ' title="'+field.descr+'" '; }
		var defaultVal = ""; if ( field.defaultVal != null ) { defaultVal = ' value="'+field.defaultVal+'" '; }

		var nameAndClass = 'id="'+divId+field.id+'" class="text ui-widget-content ui-corner-all '+divId+'PongFormField"'; 
		if ( field.type == "checkbox"  && field.name != null ) {
			nameAndClass = 'name="'+field.name+'" ' + nameAndClass; 
		} else {
			nameAndClass = 'name="'+field.id+'" ' + nameAndClass; 			
		}

		if ( field.hidden != null && field.hidden == 'true' ) { 
			nameAndClass += ' hidden'; 
		} else {
			contentItems.push( '<p><label for="'+divId+field.id+'">'+ $.i18n( field.label ) +'</label>' );
		}
		
		if ( field.type == "text" ) {
			var modifier = '';
			if ( field.readonly != null && field.readonly == 'true') { modifier += ' disabled="disabled"'; }

			if ( field.rows != null ) { 
				contentItems.push( '<textarea type="text" '+nameAndClass + title + ' rows="'+field.rows+'">'+ defaultVal +'</textarea>' );
			} else {
				contentItems.push( '<input type="text" '+nameAndClass + title + defaultVal + modifier+'/>' );				
			}
			
		} else if ( field.type == "password" ) {
			
			contentItems.push( '<input type="password" '+nameAndClass + title +'/>' );
			
		} else if ( field.type == "checkbox" ) {
			
			var cbValue = 'value="'+field.id+'" '; // default value=title
			if ( field.value != null ) { cbValue = 'value="'+field.value+'" '; }
			var modifier = '';
			if ( field.defaultVal != null && field.defaultVal == 'true') { modifier += ' checked'; }
			if ( field.readonly != null && field.readonly == 'true') { modifier += ' disabled'; }
			contentItems.push( '<input type="checkbox" '+ cbValue + nameAndClass + title + modifier +'/>' );
			
		} else if ( field.type == "checkboxList" ) {

			$.ajaxSetup({'async': false});
			$.getJSON( 
					field.resourceURL, 
					function( optData ) {
						for ( var i = 0; i < optData.length; i++ ) {
							var cbValue = 'value="'+optData[i][field.valueField]+'" '; // default value=title
							var modifier = '';
							if ( field.defaultValField != null && optData[i][field.defaultValField] == 'true') { 
								modifier += ' checked'; 
							}
							contentItems.push( '<input type="checkbox" '+ cbValue + nameAndClass + title + modifier +'/>' );
						}
					}
				);					
			$.ajaxSetup({'async': true});				

		} else if ( field.type == "select" ) {
			
			contentItems.push( '<select ' + nameAndClass  + title + '>' );
			if ( field.options != null ) {
				for ( var i = 0; i < field.options.length; i++ ) {
					contentItems.push( '<option>'+ $.i18n( field.options[i].option )+'</option>' );	
				} 		
			} else 	if ( field.optionsResource != null ) {
				$.ajaxSetup({'async': false});
				$.getJSON( 
						field.optionsResource.resourceURL, 
						function( optData ) {
							for ( var i = 0; i < optData.length; i++ ) {
								contentItems.push( '<option value="'+optData[i][ field.optionsResource.optionValue ]+'">'+
										 $.i18n( optData[i][ field.optionsResource.optionField ] ) +'</option>' );
							}
						}
					);					
				$.ajaxSetup({'async': true});				
			}

			contentItems.push( '</select>' );
			
		} else { 
			contentItems.push( '<input type="text" '+ nameAndClass  + title + defaultVal +'/>' );			
		}
		// TODO: support other form input types 
		contentItems.push( '</p>' );		
	}		
	contentItems.push( '</div>' );	
	return contentItems;
}

