/*
The MIT License (MIT)

Copyright (c) 2014 Markus Harms, ma@mh-svr.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE. 
 */
log( "Pong-Table", "load module");

var poTbl = [];

function pongTableDivHTML( divId, resourceURL, params ) {
	log( "Pong-Table",  "pongTableDivHTML: divId="+divId+" resourceURL="+resourceURL );
	poTbl[ divId ] = 
		{ 
			pongTableDef: null,
			divId: null, 
			pongTableStartRow: 0, 
			pongTableEndRow: 0,
			pongTableData: null, 
			pongTableFilter: "" 
		};
	
	poTbl[ divId ].divId = divId;
	
	if  ( moduleConfig[ divId ] != null ) {
		
		pongTableDivRenderHTML( divId, resourceURL, params, moduleConfig[ divId ] );
		
	} else {
		
		var metaURL =  resourceURL+"/pong-table";
		if ( params != null  && params.def != null ) {
			metaURL = resourceURL+"/"+params.def;
		}
		$.getJSON( metaURL, 
			function( tbl ) {
				pongTableDivRenderHTML( divId, resourceURL, params, tbl );
			}
		);		
	}
}



function pongTableDivRenderHTML( divId, resourceURL, params, tbl ) {
	log( "Pong-Table", "cre table" );
	
	var dataUrl = resourceURL;
	if ( tbl.dataURL != null ) {
		dataUrl = dataUrl+"/"+tbl.dataURL;
	}

	poTbl[ divId ].pongTableDef = tbl;
	poTbl[ divId ].resourceURL = resourceURL;
	poTbl[ divId ].pongTableDef.dataUrlFull = dataUrl
	// crunch form
	poTbl[ divId ].pongTableEndRow = tbl.maxRows;
	var contentItems = [];
	
	if ( tbl.filter != null && tbl.filter.dataReqParamsSrc != null && tbl.filter.dataReqParams != null ) {
		if ( tbl.filter.dataReqParamsSrc == 'Form' ) {
			// add filter form for table
			contentItems.push( '<div id="'+divId+'PongTableFrmDiv" class="pongTableFrm">' );
			contentItems.push( '<form id="'+divId+'PongTableFrm">' );
			contentItems.push( '<fieldset><legend>' +$.i18n('Filter') +'</legend>' );
			
			postLst = [];						
			for( var y = 0; y < tbl.filter.dataReqParams.length; y++ ) {
				prop = tbl.filter.dataReqParams[y];
				contentItems.push( '<p><label for="'+divId+prop.id+'">'+ $.i18n( prop.label ) +'</label>' );
				var nameAndClass = 'name="'+prop.id+'" id="'+divId+prop.id+'" class="text ui-widget-content ui-corner-all"'; 
				postLst.push( prop.id+": $( '#"+divId+prop.id+"' ).val()" )
				contentItems.push( '<input type="text" '+nameAndClass+'/></p>' );
				// TODO add field types
				
			}
			contentItems.push( '<button id="'+divId+'PongTableSrchBt">'+ $.i18n( 'Update Table' ) +'</button>' );
			contentItems.push( '</fieldset>' );
			contentItems.push( "</form>" );
			contentItems.push( '</div>' );


			poTbl[ divId ].pongTableFilter = postLst.join( "," );

			contentItems.push( "<script>" );
			contentItems.push( '$(function() { ' );
			contentItems.push( '    $( "#'+divId+'PongTableSrchBt" ).button().click( ' );
			contentItems.push( '       function( event ) { ' );
			contentItems.push( '           event.preventDefault(); ' );
			contentItems.push( '           udateModuleData( "'+divId+'", ' );
			contentItems.push( '             { dataFilter: { '+poTbl[ divId ].pongTableFilter+' } } ); ' );
			contentItems.push( '          return false;  ' );
			contentItems.push( '       }' );
			contentItems.push( '     );  ' );
			contentItems.push( ' }); ' );
			contentItems.push( "</script>" );
			
		} if ( tbl.filter.dataReqParamsSrc == 'SessionMap' ) {
			// TODO implement pongForm SessionMap
		}
	}
	
	contentItems.push( '<table id="'+divId+'PongTable" class="pongTable" width="100%">' );
	// cread table head
	contentItems.push( "<tr>" );
	if ( tbl.cols != null ) {
		for ( var i = 0; i < tbl.cols.length; i ++ ) {
			var colWidth = ''; if ( tbl.cols[i].width != null ) { colWidth = ' width="'+tbl.cols[i].width+'" '; }
			if ( tbl.cols[i].cellType == 'button' ) { // Button column get no headline
				contentItems.push( '<th'+colWidth+'>&nbsp;</th>'  );
			} else	if ( tbl.cols[i].cellType != 'tooltip' ) { // tool tip will be added to another col, so no own col
				contentItems.push( '<th'+colWidth+'>'+ $.i18n( (tbl.cols[i].label!=0 ? tbl.cols[i].label : '&nbsp;' ) ) +'</th>'  );
			}
		}		
	}
	contentItems.push( "</tr>" );
	for ( var r = 0; r < tbl.maxRows; r ++ ) {
		contentItems.push( "<tr>" );
		for ( var c = 0; c < tbl.cols.length; c ++ ) {
				if ( tbl.cols[c].cellType != 'tooltip' ) {
					contentItems.push( '<td id="'+divId+'R'+r+'C'+c+'">...</td>'  );
				}
		}
		contentItems.push( "</tr>" );
	}
	// paginator buttons
	contentItems.push( "</table>" );
	contentItems.push( '<button id="'+divId+'BtFirst"></button>' );
	contentItems.push( '<button id="'+divId+'BtPrev"></button>' );
	contentItems.push( '<button id="'+divId+'BtNext"></button>' );
	contentItems.push( '<button id="'+divId+'BtLast"></button>' );
	
	// paginator script
	contentItems.push( "<script>" );
	contentItems.push( "$(function() { ");
	contentItems.push( ' $( "#'+divId+'BtFirst").button( {icons:{primary:"ui-icon-arrowthickstop-1-w"}} )');
	contentItems.push( '  .click( function() { ' );
	contentItems.push( '     poTbl[ "'+divId+'" ].pongTableStartRow =0; ' );
	contentItems.push( '     poTbl[ "'+divId+'" ].pongTableEndRow = '+tbl.maxRows+';' );
	contentItems.push( '     tblCells( "'+divId+'" ); } ); ' );
	contentItems.push( ' $( "#'+divId+'BtLast" ).button( {icons:{primary:"ui-icon-arrowthickstop-1-e"}} )' );
	contentItems.push( '  .click( function() { ' );
	contentItems.push( '     poTbl[ "'+divId+'" ].pongTableStartRow =  parseInt(poTbl[ "'+divId+'" ].pongTableData.length)-parseInt('+tbl.maxRows+') ;' );
	contentItems.push( '     poTbl[ "'+divId+'" ].pongTableEndRow = poTbl[ "'+divId+'" ].pongTableData.length;' );
	contentItems.push( '     tblCells( "'+divId+'" ); } ); ' );
	
	contentItems.push( ' $( "#'+divId+'BtPrev" ).button( {icons:{primary:"ui-icon-arrowthick-1-w"}} )' );
	contentItems.push( '  .click( function() { ' );
	contentItems.push( '     if ( poTbl[ "'+divId+'" ].pongTableStartRow - '+tbl.maxRows+' >= 0 ) { ' );
	contentItems.push( '        poTbl[ "'+divId+'" ].pongTableStartRow -= '+tbl.maxRows+'; ' );
	contentItems.push( '        poTbl[ "'+divId+'" ].pongTableEndRow -= '+tbl.maxRows+';  ' );
	contentItems.push( '     } else { ' );
	contentItems.push( '        poTbl[ "'+divId+'" ].pongTableStartRow =0; ' );
	contentItems.push( '        poTbl[ "'+divId+'" ].pongTableEndRow = '+tbl.maxRows+'; ' );
	contentItems.push( '     } ' );
	contentItems.push( '     tblCells( "'+divId+'" ); } ); ' );
	
	contentItems.push( ' $( "#'+divId+'BtNext" ).button( {icons:{primary:"ui-icon-arrowthick-1-e"}} ).click( ' );
	contentItems.push( '  function() {' );
	contentItems.push( '     var xx = parseInt(poTbl[ "'+divId+'" ].pongTableStartRow) + parseInt('+tbl.maxRows +');' );
	contentItems.push( '     if ( xx < poTbl[ "'+divId+'" ].pongTableData.length ) {' );
	contentItems.push( '        poTbl[ "'+divId+'" ].pongTableStartRow = parseInt(poTbl[ "'+divId+'" ].pongTableStartRow) + parseInt('+tbl.maxRows+'); ' );
	contentItems.push( '        poTbl[ "'+divId+'" ].pongTableEndRow = parseInt(poTbl[ "'+divId+'" ].pongTableEndRow) + parseInt('+tbl.maxRows+'); ' );
	contentItems.push( '        tblCells( "'+divId+'" );' );
	contentItems.push( '      }  } ); ' );

	// AJAX commit changes for checkboxes, etc
	contentItems.push( '  $( "#'+divId+'PongTable" ).on( "change", ".postchange", ' );
	contentItems.push( '         function( event ) { ' );
	contentItems.push( '            var $this = $(this); ' );
	contentItems.push( '            var rowIdVal =  poTbl["'+divId+'"].pongTableData[ $this.data("r") ]["'+tbl.rowId+'"]; ' );
	contentItems.push( '            var colId =  poTbl["'+divId+'"].pongTableDef.cols[ $this.data("c") ].id; ' );
	contentItems.push( '            var colVal =  $this.is(":checked"); ' );
	contentItems.push( '            var postParam =  { '+tbl.rowId+': rowIdVal }; ' );
	contentItems.push( '            postParam[ colId ] =  colVal; ' );
	//contentItems.push( '            alert( "Post '+dataUrl+' "+ JSON.stringify(postParam) ); ' );
	contentItems.push( '            $.post( "'+dataUrl+'", postParam, function( response ) { }, "json");');
	contentItems.push( '            event.preventDefault(); return false; ' );
	contentItems.push( '         }' );
	contentItems.push( '  )' );

	// AJAX commit changes for editable text cells
	contentItems.push( '  $( "#'+divId+'PongTable" ).on( "mouseover", ".editableTblCell", ' );
	contentItems.push( '         function() { $(this).parent().toggleClass( "optedithighlight", true ); return $this; }' );
	contentItems.push( '  ).on( "mouseout", ".editableTblCell", ' );
	contentItems.push( '         function() { $(this).parent().toggleClass( "optedithighlight", false ); return $this; }' );
	contentItems.push( '  );' );
	contentItems.push( '  $( "#'+divId+'PongTable" ).on( "focus", ".editableTblCell", ' );
	contentItems.push( '     function() { var $this = $(this); $this.data("before", $this.html()); ' );
	contentItems.push( '        $this.parent().toggleClass( "edithighlight", true ); return $this;' );
	contentItems.push( '  }).on( "focusout", ".editableTblCell", function() { ' );
	contentItems.push( '     var $this = $(this); ' );
	contentItems.push( '     $this.parent().toggleClass( "edithighlight", false );' );
	contentItems.push( '     if ($this.data("before") !== $this.html()) {' );
	contentItems.push( '        $this.data("before", $this.html()); ' ); 
	contentItems.push( '        var rowIdVal =  poTbl["'+divId+'"].pongTableData[ $this.data("r") ]["'+tbl.rowId+'"]; ' );
	contentItems.push( '        var colId =  poTbl["'+divId+'"].pongTableDef.cols[ $this.data("c") ].id; ' );
	contentItems.push( '        var colVal =  $this.html(); ' );
	contentItems.push( '        var postParam =  { '+tbl.rowId+': rowIdVal }; ' );
	contentItems.push( '        postParam[ colId ] =  colVal; ' );
	contentItems.push( '        $.post( "'+dataUrl+'", postParam , function(response) {  }, "json");');
	//contentItems.push( '        alert( "Post Data Error: { '+tbl.rowId+': "+rowIdVal+", "+colId+": "+colVal+" }   (r="+$this.data("r") + "/c="+$this.data("c")+") not stored!!"  ); ' );
	contentItems.push( '     }' );
	contentItems.push( '     return $this;' );
	contentItems.push( '  }); ');
	contentItems.push( " }); </script>" );
 
	// create HTML
	$( "#"+divId ).html( contentItems.join( "\n" ) );
	
	if ( params != null  && params.filter != null ) {
		for ( var i = 0; i < params.filter.length; i++ ) {
			if ( i == 0 ) { 
				dataUrl += "?"+params.filter[i].field+'='+params.filter[i].value; 
			} else {
				dataUrl += "&"+params.filter[i].field+'='+params.filter[i].value; 						
			}
		}
	}
	
	// replace content of cells by data
	pongTableUpdateData( divId, null );
}

/** update data call back hook */
function pongTableUpdateData( divId, paramsObj ) {
	log( "Pong-Table",  'update '+divId );
	var tblDef = poTbl[ divId ].pongTableDef;
	
	$.getJSON( tblDef.dataUrlFull, paramsObj ,
		function( data ) { 	
			if ( tblDef.dataDocSubPath == null ) {
				// table is the root of the doc
				log( "Pong-Table",  'no tbl.dataDocSubPath' );
				poTbl[ divId ].pongTableData = data; 					
			} else {
				log( "Pong-Table",  'tbl.dataDocSubPath='+tblDef.dataDocSubPath );
				// table is somewhere in the DOM tree
				var pathToken = tblDef.dataDocSubPath.split('.');
				log( "Pong-Table",  'pathToken[0] ' + pathToken[0] );
				var subdata = data[ pathToken[0] ];
				for ( i = 1; i < pathToken.length; i++ ) {
					log( "Pong-Table", 'pathToken['+i+'] ' + pathToken[i] );	
					subdata = subdata[ pathToken[i] ];
				}
				// console.log( ' subdata = ' + JSON.stringify( subdata ) );
				poTbl[ divId ].pongTableData = subdata;
			}
			tblCells( divId ); 
		} 
	);	
}


/** render table cells */
function tblCells( divId ) {
	var rowSt = parseInt( poTbl[ divId ].pongTableStartRow );
	var rowEn = parseInt( poTbl[ divId ].pongTableEndRow );
	log( "Pong-Table", "tblCells: divId="+divId+"Data #"+poTbl[ divId ].pongTableData.length + " rowSt="+rowSt + " rowEn="+rowEn );
	var i = 0;
	for ( var r = rowSt; r < rowEn; r++ ) {
		if ( r < poTbl[ divId ].pongTableData.length ) {
			var cellDta = poTbl[ divId ].pongTableData[r];
			for ( var c = 0; c < poTbl[ divId ].pongTableDef.cols.length; c++ ) {
				var cellDef = poTbl[ divId ].pongTableDef.cols[c];
				log( "Pong-Table", cellDef );
				var cellId =  '#'+divId+'R'+i+'C'+c; 
				var cellType = cellDef.cellType;
				var editable = '';	
				log( "Pong-Table", "R="+i+" C="+c+" "+cellType+ "  "+cellDta[ cellDef.id ] );
				if ( cellType == 'text' ) {
					if ( ( cellDef.editable != null ) && ( cellDef.editable == "true" ) ) { 
						editable = 'contenteditable="true" class="editableTblCell" data-r="'+r+'" data-c="'+c+'"'; 
						$( cellId ).html( '<div style="position:relative" class="editable"><span id="'+divId+'R'+i+cellDef.id+'" '+editable+'>'+cellDta[ cellDef.id ] + '</span><div class="ui-icon ui-icon-pencil editmarker"></div></div>' );
					} else {
						$( cellId ).html( '<span id="'+divId+'R'+i+cellDef.id+'">'+cellDta[ cellDef.id ] + '</span>' );
					}
				} else if ( cellType == 'checkbox' ) {
					if ( ( cellDef.editable != null ) && ( cellDef.editable == "true" ) ) {
						editable = 'class="postchange"  data-r="'+r+'" data-c="'+c+'"';
					} else { editable = 'disabled' };
					if ( cellDta[ cellDef.id ]=="true" || cellDta[ cellDef.id ]==true ) {
						$( cellId ).html( '<input type="checkbox" '+editable+' value="'+cellDef.id+'" id="'+divId+'R'+i+cellDef.id+'" checked />' );												
					} else {
						$( cellId ).html( '<input type="checkbox" '+editable+' value="'+cellDef.id+'" id="'+divId+'R'+i+cellDef.id+'"/>' );
					}
				} else if ( cellType == 'linkLink' ) {
					$( cellId ).html( '<a href="'+cellDta[ cellDef.id ]+'" id="'+divId+'R'+i+cellDef.id+'">'+$.i18n( cellDef.label )+'</a>' );
				} else if ( cellType == 'img' ) {
					$( cellId ).html( '<img src="'+cellDta[ cellDef.id ]+'" id="'+divId+'R'+i+cellDef.id+'"/>' );
				} else if ( cellType == 'button'  ) {
					var contentItems = [];
					var ajaxType = 'POST';
					var param = '';
					if ( ( cellDef.method != null ) && ( cellDef.method.length != null ) ) {
						if ( cellDef.method == 'DEL-POST' ) {
							param = 'actn:"DEL",';
						} else {
							ajaxType = cellDef.method; 
						}
					} 
					var url = poTbl[ divId ].resourceURL+'/?'+poTbl[ divId ].pongTableDef.rowId+'='+cellDta[ poTbl[ divId ].pongTableDef.rowId ];
					if ( ( cellDef.url != null ) && ( cellDef.url.length != null ) ) {
						url = cellDef.url;
					}
					contentItems.push( '<button id="'+divId+'R'+i+cellDef.id+'">'+cellDef.label+'</button>' );
					contentItems.push( '<script>' );
					contentItems.push( '  $( function() { ' );
					contentItems.push( '       $( "#' +divId+'R'+i+cellDef.id+ '" ).click(' );
					contentItems.push( '          function() {  '); //alert( "'+ajaxType+' data: { rowId : '+ poTbl[ divId ].pongTableDef.rowId+'='+cellDta[ poTbl[ divId ].pongTableDef.rowId ] +' }"); ' );
					contentItems.push( '              $.ajax( ' );
					contentItems.push( '                 { url: "'+url+'", ');
					contentItems.push( '                   type: "'+ajaxType+'", ' );
					contentItems.push( '                   data: { rowId : "'+ cellDta[ poTbl[ divId ].pongTableDef.rowId ] +'"'+param+' } } ' );
					contentItems.push( '              ).done(  ' );
					contentItems.push( '                 function( dta ) { '); // alert( dta ); ' );
					if ( cellDef.target != null ) {
						if ( cellDef.target == '_parent' ) {
							contentItems.push( '                       window.location.replace( dta );');
						} else if ( cellDef.target == 'modal' ) {
							contentItems.push( '                       alert( dta );  ' );
						} else {
							contentItems.push( '                       $( "#'+cellDef.target+'Content" ).html( dta );  ' );									
						}
					}
					if ( ( cellDef.update != null ) && ( cellDef.update.length != null ) ) {
						for ( var upCnt = 0; upCnt < cellDef.update.length; upCnt++ ) {
							if ( cellDef.method == 'DELETE' ) {
								contentItems.push( '                 udateModuleData( "'+cellDef.update[upCnt].resId+'Content", { }  ); ' ); // otherwise deleted ID is requested and result is empty 					
							} else {
								contentItems.push( '                 udateModuleData( "'+cellDef.update[upCnt].resId+'Content", { "'+ poTbl[ divId ].pongTableDef.rowId + '": "'+ cellDta[ poTbl[ divId ].pongTableDef.rowId ] +'" }  ); ' ); 													
							}
						}
					}
					contentItems.push( '                       return false;' ); 
					contentItems.push( '                  }  ' );
					contentItems.push( '              ); ');
					contentItems.push( '              return false;' ); 
					contentItems.push( '          }' );
					contentItems.push( '       ); ' );

					contentItems.push( '  } ); ' ); 
					contentItems.push( '</script>' );
					$( cellId ).html( contentItems.join( "\n" ) );
				} else if ( cellType == 'tooltip'  ) {
					$( '#'+divId+'R'+i+cellDef.label ).attr( 'title' , cellDta[ cellDef.id ] );
				} else {
					// ???
				}	
			}
		} else { // clear the rest of the cells
			for ( var c = 0; c < poTbl[ divId ].pongTableDef.cols.length; c++ ) {
				var cellId =  '#'+divId+'R'+i+'C'+c; 
				$( cellId ).html( '&nbsp;' );
			}
		}
		i++;
	}	
}
 